@@ -344,6 +344,7 @@ class SeedHypervisorHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
     * Configure the host's network interfaces.
     * Set sysctl parameters.
     * Configure NTP.
+    * Optionally, configure software RAID arrays.
     * Configure LVM volumes.
     * Configure the host as a libvirt hypervisor.
     """
@@ -380,12 +381,39 @@ class SeedHypervisorHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
         if parsed_args.wipe_disks:
             playbooks += _build_playbook_list("wipe-disks")
         playbooks += _build_playbook_list(
-            "users", "yum", "dev-tools", "network", "sysctl", "ntp", "lvm",
-            "seed-hypervisor-libvirt-host")
+            "users", "yum", "dev-tools", "network", "sysctl", "ntp", "mdadm",
+            "lvm", "seed-hypervisor-libvirt-host")
         self.run_kayobe_playbooks(parsed_args, playbooks,
                                   limit="seed-hypervisor")
 
 
+class SeedHypervisorHostPackageUpdate(KayobeAnsibleMixin, VaultMixin, Command):
+    """Update packages on the seed hypervisor host."""
+
+    def get_parser(self, prog_name):
+        parser = super(SeedHypervisorHostPackageUpdate, self).get_parser(
+            prog_name)
+        group = parser.add_argument_group("Host Package Updates")
+        group.add_argument("--packages", required=True,
+                           help="List of packages to update. Use '*' to "
+                                "update all packages.")
+        group.add_argument("--security", action='store_true',
+                           help="Only install updates that have been marked "
+                                "security related.")
+        return parser
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Updating seed hypervisor host packages")
+        extra_vars = {
+            "host_package_update_packages": parsed_args.packages,
+            "host_package_update_security": parsed_args.security,
+        }
+        playbooks = _build_playbook_list("host-package-update")
+        self.run_kayobe_playbooks(parsed_args, playbooks,
+                                  limit="seed-hypervisor",
+                                  extra_vars=extra_vars)
+
+
 class SeedHypervisorHostCommandRun(KayobeAnsibleMixin, VaultMixin, Command):
     """Run command on the seed hypervisor host."""
 
@@ -472,6 +500,7 @@ class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
     * Configure IP routing and source NAT.
     * Disable bootstrap interface configuration.
     * Configure NTP.
+    * Optionally, configure software RAID arrays.
     * Configure LVM volumes.
     * Optionally, create a virtualenv for kolla-ansible.
     * Configure a user account for kolla-ansible.
@@ -520,7 +549,8 @@ class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
             playbooks += _build_playbook_list("wipe-disks")
         playbooks += _build_playbook_list(
             "users", "yum", "dev-tools", "disable-selinux", "network",
-            "sysctl", "ip-routing", "snat", "disable-glean", "ntp", "lvm")
+            "sysctl", "ip-routing", "snat", "disable-glean", "ntp", "mdadm",
+            "lvm")
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="seed")
 
         self.generate_kolla_ansible_config(parsed_args, service_config=False)
@@ -674,8 +704,7 @@ class SeedServiceUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         playbooks = _build_playbook_list(
             "seed-service-upgrade-prep")
         self.run_kayobe_playbooks(parsed_args, playbooks)
-
-        self.run_kolla_ansible_seed(parsed_args, "deploy-bifrost")
+        self.run_kolla_ansible_seed(parsed_args, "upgrade-bifrost")
         playbooks = _build_playbook_list(
             "overcloud-host-image-workaround-resolv",
             "overcloud-host-image-workaround-cloud-init",
@@ -872,6 +901,7 @@ class OvercloudHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
     * Set sysctl parameters.
     * Disable bootstrap interface configuration.
     * Configure NTP.
+    * Optionally, configure software RAID arrays.
     * Configure LVM volumes.
     * Optionally, create a virtualenv for kolla-ansible.
     * Configure a user account for kolla-ansible.
@@ -919,7 +949,8 @@ class OvercloudHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
             playbooks += _build_playbook_list("wipe-disks")
         playbooks += _build_playbook_list(
             "users", "yum", "dev-tools", "disable-selinux", "network",
-            "sysctl", "disable-glean", "disable-cloud-init", "ntp", "lvm")
+            "sysctl", "disable-glean", "disable-cloud-init", "ntp", "mdadm",
+            "lvm")
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud")
 
         self.generate_kolla_ansible_config(parsed_args, service_config=False)
@@ -950,7 +981,7 @@ class OvercloudHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         # Further kayobe playbooks.
         playbooks = _build_playbook_list(
             "pip", "kolla-target-venv", "kolla-host",
-            "docker", "ceph-block-devices")
+            "docker", "ceph-block-devices", "swift-block-devices")
         self.run_kayobe_playbooks(parsed_args, playbooks,
                                   extra_vars=extra_vars, limit="overcloud")
 
@@ -1014,6 +1045,49 @@ class OvercloudHostUpgrade(KayobeAnsibleMixin, VaultMixin, Command):
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud")
 
 
+class OvercloudDatabaseBackup(KollaAnsibleMixin, VaultMixin, Command):
+    """Backup the overcloud database."""
+
+    def get_parser(self, prog_name):
+        parser = super(OvercloudDatabaseBackup, self).get_parser(prog_name)
+        group = parser.add_argument_group("Overcloud Database Backup")
+        group.add_argument("--incremental", action='store_true',
+                           help="Whether to perform an incremental database "
+                                "backup. Default is false.")
+        return parser
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Performing overcloud database backup")
+        extra_args = []
+        if parsed_args.incremental:
+            extra_args.append('--incremental')
+        self.run_kolla_ansible_overcloud(parsed_args, "mariadb_backup",
+                                         extra_args=extra_args)
+
+
+class OvercloudDatabaseRecover(KollaAnsibleMixin, VaultMixin, Command):
+    """Recover the overcloud database."""
+
+    def get_parser(self, prog_name):
+        parser = super(OvercloudDatabaseRecover, self).get_parser(prog_name)
+        group = parser.add_argument_group("Overcloud Database Recovery")
+        group.add_argument("--force-recovery-host",
+                           help="Name of a host to use to perform the "
+                                "recovery. By default kolla-ansible will "
+                                "automatically determine which host to use, "
+                                "and this option should not be used.")
+        return parser
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Performing overcloud database recovery")
+        extra_vars = {}
+        if parsed_args.force_recovery_host:
+            extra_vars['mariadb_recover_inventory_name'] = (
+                parsed_args.force_recovery_host)
+        self.run_kolla_ansible_overcloud(parsed_args, "mariadb_recovery",
+                                         extra_vars=extra_vars)
+
+
 class OvercloudServiceConfigurationGenerate(KayobeAnsibleMixin,
                                             KollaAnsibleMixin, VaultMixin,
                                             Command):
@@ -1044,9 +1118,6 @@ class OvercloudServiceConfigurationGenerate(KayobeAnsibleMixin,
         # First prepare configuration.
         self.generate_kolla_ansible_config(parsed_args)
 
-        playbooks = _build_playbook_list("swift-setup")
-        self.run_kayobe_playbooks(parsed_args, playbooks)
-
         # Run kolla-ansible prechecks before deployment.
         if not parsed_args.skip_prechecks:
             self.run_kolla_ansible_overcloud(parsed_args, "prechecks")
@@ -1132,9 +1203,6 @@ class OvercloudServiceDeploy(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         # First prepare configuration.
         self.generate_kolla_ansible_config(parsed_args)
 
-        playbooks = _build_playbook_list("swift-setup")
-        self.run_kayobe_playbooks(parsed_args, playbooks)
-
         # Run kolla-ansible prechecks before deployment.
         if not parsed_args.skip_prechecks:
             self.run_kolla_ansible_overcloud(parsed_args, "prechecks")
@@ -1193,9 +1261,6 @@ class OvercloudServiceReconfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
         # First prepare configuration.
         self.generate_kolla_ansible_config(parsed_args)
 
-        playbooks = _build_playbook_list("swift-setup")
-        self.run_kayobe_playbooks(parsed_args, playbooks)
-
         # Run kolla-ansible prechecks before reconfiguration.
         if not parsed_args.skip_prechecks:
             self.run_kolla_ansible_overcloud(parsed_args, "prechecks")
@@ -1408,6 +1473,15 @@ class OvercloudPostConfigure(KayobeAnsibleMixin, VaultMixin, Command):
         self.run_kayobe_playbooks(parsed_args, playbooks)
 
 
+class OvercloudSwiftRingsGenerate(KayobeAnsibleMixin, VaultMixin, Command):
+    """Generate Swift rings."""
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Generating Swift rings")
+        playbooks = _build_playbook_list("swift-rings")
+        self.run_kayobe_playbooks(parsed_args, playbooks)
+
+
 class NetworkConnectivityCheck(KayobeAnsibleMixin, VaultMixin, Command):
     """Check network connectivity between hosts in the control plane.
 
